#ifndef ALBENDETAIL_H
#define ALBENDETAIL_H

#include "album.h"
#include "detailmodi.h"

#include <QMainWindow>
#include <QSettings>
#include <QString>
#include <QVector>

namespace Ui {
	class AlbenDetail;
}

/**
 * @brief Window for entry details
 *
 * showing for new entry, view details, change entry
 */
class AlbenDetail : public QMainWindow
{
		Q_OBJECT

	public:
		explicit AlbenDetail(QWidget *parent = nullptr);
		~AlbenDetail();		
		void load(QString ean, DetailModi mode);

	protected:
		void closeEvent(QCloseEvent *envent);

	private slots:
		void addEntry();
		void btnBackClick();
		void changeEntry();
		void comboLocationTextChanged(QString newText);

	private:
		/**
		 * @brief link to the user interface
		 */
		Ui::AlbenDetail *ui;

		/**
		 * @brief the table to look in
		 */
		const QString TABLE = "Albums";

		/**
		 * @brief primary key of @see TABLE
		 */
		const QString PRIMARY_KEY = "EAN";

		/**
		 * @brief columns in the order in wich they can be find in @see TABLE
		 */
		const QVector<QString> COLUMNS{"EAN", "Title", "Interpret", "Year", "Genre", "Location", "Type", "Owner" };

		QSettings settings;

		void disableAll();
		void enableAll();
		void getEntry(QString ean);
		void output(const Album *album);
		void setEanReadOnly();
		bool tryCreateDatabaseEntry(QMap<QString, QString> &databaseEntry);
		void setMediaType(bool isCD_DVD);
};

#endif // ALBENDETAIL_H
