#ifndef ALBENLIST_H
#define ALBENLIST_H

#include <QList>
#include <QMainWindow>
#include <QString>
#include <QVector>

#include "smallalbum.h"

namespace Ui {
	class AlbenList;
}

class AlbenList : public QMainWindow
{
		Q_OBJECT

	public:
		explicit AlbenList(QWidget *parent = nullptr);
		~AlbenList();

	protected:
		void showEvent(QShowEvent *ev);

	private slots:
		void addEntry();
		void changeEntry();
		void changeView();
		void deleteEntry();
		void lstSelectedIndexChanged(int index);
		void searchCriterionChanged();
		void viewEntry();
		void windowLoaded();
		void showOnlineSearch();

	private:
		/**
		 * @brief the user-interface
		 */
		Ui::AlbenList *ui;

		/**
		 * @brief name of database table to look in
		 */
		const QString TABLE = "Albums";

		/**
		 * @brief sort entrys by these column [by default]
		 */
		const QString SORTING_CRETERIA = "Interpret, Title";

		/**
		 * @brief EANs of all shown database entrys
		 *
		 * need to identify entrys in @see lstValues
		 */
		QVector<QString> eans{};

		void deleteEntry(QString ean);
		void disableButtons();
		void enableButtons();
		void output(QList<SmallAlbum*> *albums);
		void search(QMap<QString, QString> *searchFilter = nullptr);

};

#endif // ALBENLIST_H
