#ifndef TITLELIST_H
#define TITLELIST_H

#include <QMainWindow>
#include <QString>
#include <QVector>

#include "smalltitle.h"

namespace Ui {
	class TitleList;
}

class TitleList : public QMainWindow
{
		Q_OBJECT

	public:
		explicit TitleList(QWidget *parent = nullptr);
		~TitleList();

	protected:
		void showEvent(QShowEvent *ev);

	private slots:
		void addEntry();
		void changeEntry();
		void changeView();
		void deleteEntry();
		void lstSelectedIndexChanged(int index);
		void searchCriterionChanged();
		void viewEntry();
		void windowLoaded();

	private:
		/**
		 * @brief The UI
		 */
		Ui::TitleList *ui;

		/**
		 * @brief name of database table to look in
		 */
		const QString TABLE = "Titles";

		/**
		 * @brief sort entrys by these column [by default]
		 */
		const QString SORTING_CRETERIA = "Interpret, Album, Title";

		/**
		 * @brief IDs of all shown database entrys
		 *
		 *  need to identify entrys in @see lstValues
		 */
		QVector<int> ids{};

		QString createCriterion(bool album, bool interpret, bool title, bool year);
		void deleteEntry(int id);
		void disableButtons();
		void enableButtons();
		void output(QList<SmallTitle*> *titles);
		void search(QMap<QString,QString> *filter = nullptr);
};

#endif // TITLELIST_H
