#include "albendetail.h"
#include "ui_albendetail.h"

#include "album.h"
#include "dataworker.h"
#include "definitions.h"
#include "server.h"

#include <SqlWorker/DatabaseWorker.hpp>

#include <QMap>
#include <QMessageBox>
#include <QSet>
#include <QSettings>

/**
 * @brief create a new AlbenDetail obejct
 * @param parent
 */
AlbenDetail::AlbenDetail(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::AlbenDetail),
	settings()
{
	ui->setupUi(this);

	int size = settings.beginReadArray("owners");
	for (int var = 0; var < size; ++var)
	{
		settings.setArrayIndex(var);
		ui->comboOwn->addItem(settings.value("owner").toString());
	}
	settings.endArray();

	size = settings.beginReadArray("locations");
	for (int var = 0; var < size; ++var)
	{
		settings.setArrayIndex(var);
		ui->comboLoc->addItem(settings.value("location").toString());
	}
	settings.endArray();
}

/**
 * @brief delete the user interface
 */
AlbenDetail::~AlbenDetail()
{
	delete ui;
}

/**
 * @brief adds a new entry to the database
 */
void AlbenDetail::addEntry()
{
	QMap<QString,QString> entry;
	if (tryCreateDatabaseEntry(entry))
	{
		if (Server::getInstance().database->insertNewEntry(TABLE, entry))
		{
			QMessageBox::information(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						QApplication::translate("Definitions", INSERT_SUCESS),
						QMessageBox::Ok,
						QMessageBox::Ok);
			this->close();
		}
		else
		{
			QMessageBox::critical(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						Server::getInstance().database->DatabaseWorker::getLastError(),
						QMessageBox::Ok,
						QMessageBox::Ok);
		}
	}
	else
	{
		QMessageBox::warning(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					QApplication::translate("Definitions", CHANGE_SUCCESS),
					QMessageBox::Ok,
					QMessageBox::Ok);
	}
}

/**
 * @brief slot for 'back' button, close this window
 */
void AlbenDetail::btnBackClick()
{
	this->close();
}

/**
 * @brief saves changes on a entry to @see TABLE
 */
void AlbenDetail::changeEntry()
{
	QMap<QString,QString> entry;
	if (tryCreateDatabaseEntry(entry))
	{
		QMap<QString,QString> filter{};
		filter.insert("EAN", ui->txtEAN->text());

		if (Server::getInstance().database->updateEntries(TABLE, filter, entry))
		{
			QMessageBox::information(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						QApplication::translate("Definitions", CHANGE_SUCCESS),
						QMessageBox::Ok,
						QMessageBox::Ok);
			this->close();
		}
		else
		{
			QMessageBox::critical(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						Server::getInstance().database->DatabaseWorker::getLastError(),
						QMessageBox::Ok,
						QMessageBox::Ok);
		}
	}
	else
	{
		QMessageBox::warning(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					QApplication::translate("Definitions", CREATION_FAILED),
					QMessageBox::Ok,
					QMessageBox::Ok);
	}
}

/**
 * @brief prepares window for 'view entry only' mode
 */
void AlbenDetail::disableAll()
{
	ui->btnAdd->setEnabled(false);
	ui->btnBack->setDefault(true);
	ui->btnSubmit->setEnabled(false);
	ui->comboLoc->setEnabled(false);
	ui->comboOwn->setEnabled(false);
	ui->radioCD->setEnabled(false);
	ui->radioDVD->setEnabled(false);
	ui->txtEAN->setReadOnly(true);
	ui->txtGrene->setReadOnly(true);
	ui->txtIpret->setReadOnly(true);
	ui->txtTitle->setReadOnly(true);
	ui->txtYear->setReadOnly(true);
}

/**
 * @brief prepares window for 'add entry' mode
 */
void AlbenDetail::enableAll()
{
	ui->btnAdd->setEnabled(true);
	ui->btnAdd->setDefault(true);
	ui->btnSubmit->setEnabled(true);
	ui->comboLoc->setEnabled(true);
	ui->comboOwn->setEnabled(true);
	ui->radioCD->setEnabled(true);
	ui->radioDVD->setEnabled(true);
	ui->txtEAN->setReadOnly(false);
	ui->txtGrene->setReadOnly(false);
	ui->txtIpret->setReadOnly(false);
	ui->txtTitle->setReadOnly(false);
	ui->txtYear->setReadOnly(false);
}

void AlbenDetail::closeEvent(QCloseEvent *event)
{
	QSet<QString> owners;
	for (int i = 0; i < ui->comboOwn->count(); ++i)
	{
		QString owner = ui->comboOwn->itemText(i).simplified();
		if (owner != "") { owners.insert(owner); }
	}
	owners.insert(ui->comboOwn->currentText());

	settings.beginWriteArray("owners");
	int owner_counter = 0;
	foreach (QString elem, owners)
	{
		settings.setArrayIndex(owner_counter);
		settings.setValue("owner", elem);
		owner_counter++;
	}
	settings.endArray();

	QSet<QString> locations;
	for (int i = 0; i < ui->comboLoc->count(); ++i)
	{
		QString location = ui->comboLoc->itemText(i).simplified();
		if (location != "") { locations.insert(location); }
	}
	locations.insert(ui->comboLoc->currentText());

	settings.beginWriteArray("locations");
	int location_counter = 0;
	foreach (QString elem, locations)
	{
		settings.setArrayIndex(location_counter);
		settings.setValue("location", elem);
		location_counter++;
	}
	settings.endArray();

	QMainWindow::closeEvent(event);
}

/**
 * @brief gets an entry from database
 * @param ean identifier [primary key] of the searched entry
 */
void AlbenDetail::getEntry(QString ean)
{
	QMap<QString,QString> filter{};
	filter.insert("EAN", ean);

	QList<QVector<QString>*> *dbValue =
			Server::getInstance().database->searchEntries(TABLE, &filter, nullptr, PRIMARY_KEY);

	if (dbValue != nullptr && dbValue->count() > 0)
	{
		Album *album = DataWorker::getAlbum(dbValue->front());
		if (album != nullptr)
		{
			output(album);
			delete album;
		}
		else
		{
			QMessageBox::critical(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						QApplication::translate("Definitions", DATA_READ_ERROR)
			);
		}
		delete dbValue;
	}
	else
	{
		QMessageBox msgbox;
		msgbox.setText(DB_READ_ERROR);
		msgbox.setInformativeText(Server::getInstance().database->DatabaseWorker::getLastError());
		msgbox.setIcon(QMessageBox::Critical);
		msgbox.exec();
	}


}

/**
 * @brief set values based on \p ean, showing window
 * @param ean ean of the entry to select from database
 * @param mode defines how the window should look like
 */
void AlbenDetail::load(QString ean, DetailModi mode)
{
	switch (mode)
	{
		case DetailModi::MODE_ADD_ENTRY:
			enableAll();
			ui->btnAdd->setDefault(true);
			break;

		case DetailModi::MODE_READONLY_ALL:
			getEntry(ean);
			disableAll();
			ui->btnBack->setDefault(true);
			break;

		case DetailModi::MODE_READONLY_IDENTIFYER:
			getEntry(ean);
			setEanReadOnly();
			ui->btnSubmit->setDefault(true);
			break;

		default:
			getEntry(ean);
			break;
	}
	this->show();
}

/**
 * @brief prints all @see Album inforamtions
 * @param album the given album
 */
void AlbenDetail::output(const Album *album)
{
	if (album != nullptr)
	{
		ui->txtEAN->setText(album->ean);
		ui->txtGrene->setText(album->grene);
		ui->txtIpret->setText(album->interpret);
		ui->txtTitle->setText(album->title);
		ui->txtYear->setText(QString::number(album->year));
		//ui->radioDVD->setChecked(album->isCD_DVD);
		ui->comboLoc->setCurrentText(album->location);
		ui->comboOwn->setCurrentText(album->owner);
		setMediaType(album->isCD_DVD);
	}
}

void AlbenDetail::setMediaType(bool is_CD_DVD)
{

	ui->radioCD->setChecked(!is_CD_DVD);
	ui->radioDVD->setChecked(is_CD_DVD);
}

/**
 * @brief prepares form for 'edit entry' mode
 */
void AlbenDetail::setEanReadOnly()
{
	ui->btnAdd->setEnabled(false);
	ui->btnSubmit->setEnabled(true);
	ui->btnSubmit->setDefault(true);
	ui->comboLoc->setEnabled(true);
	ui->comboOwn->setEnabled(true);
	ui->radioCD->setEnabled(true);
	ui->radioDVD->setEnabled(true);
	ui->txtEAN->setReadOnly(true);
	ui->txtGrene->setReadOnly(false);
	ui->txtIpret->setReadOnly(false);
	ui->txtTitle->setReadOnly(false);
	ui->txtYear->setReadOnly(false);
}

/**
 * @brief creates the string-array for database out of form-control-values
 * @param databaseEntry filled map for database as output
 * @return <tt>true</tt> if creation was successfull, <tt>false</tt> otherwise
 */
bool AlbenDetail::tryCreateDatabaseEntry(QMap<QString,QString> &databaseEntry)
{

	QString ean = ui->txtEAN->text();

	bool ok = true;
	int year = ui->txtYear->text().toInt(&ok);

	if (!ok)
	{
		QMessageBox::critical(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					QApplication::translate("Definitions", CREATION_FAILED)
		);
		return false;
	}

	Album album(ean,
				ui->txtTitle->text(),
				ui->txtIpret->text(),
				year,
				ui->txtGrene->text(),
				ui->comboLoc->currentText().simplified(),
				ui->radioDVD->isChecked(),
				ui->comboOwn->currentText().simplified());

	databaseEntry = DataWorker::getAlbumMapForDatabase(&album);
	return true;
}

/**
 * @brief The text of ui->comboLoc has changed
 * @param text The new text of the control
 *
 * This function checks whether the maximum input length of this field
 * has been exceeded. If this is the case, we truncate the right most
 * characters to fit to the value of 13 + one space character.
 *
 * <b>Note:</b> This function does not manually insert the first space
 * character if it was deleted. The user has to carefully update this field.
 * Otherwise the first character will be deleted during further processing
 * of this value.
 */
void AlbenDetail::comboLocationTextChanged(QString text)
{
	if (text.count() > 14)
	{
		ui->comboLoc->setCurrentText(text.left(14));
	}
}
