#include "albenlist.h"
#include "ui_albenlist.h"

#include "albendetail.h"
#include "albentable.h"
#include "dataworker.h"
#include "definitions.h"
#include "detailmodi.h"
#include "server.h"
#include "titlelist.h"
#include "titletable.h"

#include <SqlWorker/DatabaseWorker.hpp>

#include <QMap>
#include <QMessageBox>
#include <QString>
#include <QVector>

#include <UmlauteWeb/Qt.hpp>


#ifdef ONLINE_SUPPORT
#include "onlinesearchwindow.h"
#endif

/**
 * @brief constructor, initializing UI
 * @param parent
 */
AlbenList::AlbenList(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::AlbenList)
{
	ui->setupUi(this);

#ifndef ONLINE_SUPPORT
	ui->btnOnlineSearch->setEnabled(false);
#endif
}

/**
 * @brief Destrucotr, destruct UI
 */
AlbenList::~AlbenList()
{
	delete ui;
}

/**
 * @brief showing @see AlbenDetail in 'add entry' mode
 */
void AlbenList::addEntry()
{
	QString ean = "-1";
	DetailModi mode = DetailModi::MODE_ADD_ENTRY;
	AlbenDetail *albumDetail = new AlbenDetail();
	albumDetail->setAttribute(Qt::WA_DeleteOnClose);
	albumDetail->load(ean, mode);
}

/**
 * @brief changing the selectes entry by showing @see AlbenDetail in 'change' mode
 */
void AlbenList::changeEntry()
{
	int index = ui->lstValues->currentRow();
	QString ean = eans[index];
	DetailModi mode = DetailModi::MODE_READONLY_IDENTIFYER;
	AlbenDetail *albumDetail = new AlbenDetail();
	albumDetail->setAttribute(Qt::WA_DeleteOnClose);
	albumDetail->load(ean, mode);
}

/**
 * @brief change to another view
 */
void AlbenList::changeView()
{
	if (sender()->objectName() == "actionAlbenFull")
	{
		AlbenTable *albenTable = new AlbenTable();
		albenTable->setAttribute(Qt::WA_DeleteOnClose);
		albenTable->show();
		//close();
	}

	if (sender()->objectName() == "actionTitelStd")
	{
		TitleList *titleList = new TitleList();
		titleList->setAttribute(Qt::WA_DeleteOnClose);
		titleList->show();
		//close();
	}

	if (sender()->objectName() == "actionTitelFull")
	{
		TitleTable *titleTable = new TitleTable();
		titleTable->setAttribute(Qt::WA_DeleteOnClose);
		titleTable->show();
		//close();
	}
	close();
}

/**
 * @brief user action: click on delete
 */
void AlbenList::deleteEntry()
{
	int index = ui->lstValues->currentIndex().row();
	QString ean = eans[index];
	deleteEntry(ean);
}

/**
 * @brief delete entry from database
 * @param ean identify the entry to delete
 */
void AlbenList::deleteEntry(QString ean)
{
	QMessageBox::StandardButton requestNo = QMessageBox::question(
												this,
												QApplication::translate("Definitions", PRODUCT_NAME),
												QApplication::translate("Definitions", ARE_YOU_SURE_DELETE_ENTRY)
											);

	if (requestNo == QMessageBox::StandardButton::No) { return; }

	//bool deleteOK = Server::getInstance().database->DatabaseWorker::deleteEntry(ean, TABLE);
	QMap<QString, QString> filter{};
	filter.insert("EAN", ean);
	bool deleteOK = Server::getInstance().database->deleteEntry(TABLE, filter);

	if (deleteOK)
	{
		QMessageBox::information(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					QApplication::translate("Definitions", DELETE_SUCCESS)
		);
	}
	else
	{
		QMessageBox msgBox;
		msgBox.setText(QApplication::translate("Definitions", DELETE_FAILED));
		msgBox.setWindowTitle(QApplication::translate("Definitions", PRODUCT_NAME));
		msgBox.setInformativeText(Server::getInstance().database->DatabaseWorker::getLastError());
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setIcon(QMessageBox::Icon::Critical);
		msgBox.exec();
	}
	search(nullptr);
}

/**
 * @brief disables options change, delete, modify
 */
void AlbenList::disableButtons()
{
	ui->actionChange->setEnabled(false);
	ui->actionView->setEnabled(false);
	ui->actionDelete->setEnabled(false);
	ui->btnDel->setEnabled(false);
	ui->btnEdit->setEnabled(false);
	ui->btnView->setEnabled(false);
}

/**
 * @brief enables options change, delete, modify
 */
void AlbenList::enableButtons()
{
	ui->actionChange->setEnabled(true);
	ui->actionView->setEnabled(true);
	ui->actionDelete->setEnabled(true);
	ui->btnDel->setEnabled(true);
	ui->btnEdit->setEnabled(true);
	ui->btnView->setEnabled(true);
}

/**
 * @brief check if buttons need to be activated
 * @param index the current selected index of @see ui->lstValues
 */
void AlbenList::lstSelectedIndexChanged(int index)
{
	if (index > -1) { enableButtons(); }
	else { disableButtons(); }
}

/**
 * @brief displays the given \p albums
 * @param albums the entrys to display in LstValues
 */
void AlbenList::output(QList<SmallAlbum*> *albums)
{
	ui->lstValues->clear();
	eans.clear();

	if (albums != nullptr)
	{
		for (auto &&album : *albums)
		{
			QString titleStr = album->title;
			Umlaute_Web::Qt::decode(titleStr);

			QString interpretStr = album->interpret;
			Umlaute_Web::Qt::decode(interpretStr);

			QString outputStr = titleStr + " # " + interpretStr;
			ui->lstValues->addItem(outputStr);
			eans.append(album->ean);
		}
		disableButtons();
	}
}

/**
 * @brief search for a specific entry
 * @param creteria the row to look in
 * @param value the value to search for
 */
void AlbenList::search(QMap<QString,QString> *searchFilter)
{
	// create the list of cloumns
	QVector<QString> columns = {"EAN", "Title", "Interpret"};

	QList<QVector<QString>*> *res = nullptr;

	// Get results from the database; sorting along interpret and title
	res = Server::getInstance().database->searchEntries(TABLE, searchFilter, &columns, "Interpret, Title");

	// error from MySQLWorker?
	if (res == nullptr)
	{
		QMessageBox::critical(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					Server::getInstance().database->DatabaseWorker::getLastError(),
					QMessageBox::Ok);
		return;
	}
	QList<SmallAlbum*> *results = DataWorker::getSmallAlbums(res);
	output(results);

	if (searchFilter != nullptr) { delete searchFilter; }
	delete results;
	delete res;

}

/**
 * @brief filteroptions changed => new search
 */
void AlbenList::searchCriterionChanged()
{
	search(DataWorker::createSearchFilterForAlbum(ui->comboSearch1->currentIndex(),
												  ui->comboSearch2->currentIndex(),
												  ui->txtSearch1->text(),
												  ui->txtSearch2->text()));
}

/**
 * @brief doing stuff while window ist beginning to be visible
 * @param ev the underlaying event
 */
void AlbenList::showEvent(QShowEvent *ev)
{
	QMainWindow::showEvent(ev);
	QMetaObject::invokeMethod(this, "windowLoaded", Qt::ConnectionType::QueuedConnection);
}

/**
 * @brief view details of the selected entry by showing @see AlbenDetail in 'readonly' mode
 */
void AlbenList::viewEntry()
{
	int index = ui->lstValues->currentRow();
	QString ean = eans[index];
	DetailModi mode = DetailModi::MODE_READONLY_ALL;
	AlbenDetail *albumDetail = new AlbenDetail();
	albumDetail->setAttribute(Qt::WA_DeleteOnClose);
	albumDetail->load(ean, mode);
}

/**
 * @brief when window is loading show all entrys
 */
void AlbenList::windowLoaded()
{
	search(nullptr);
}

/**
 * @brief Show the online search window
 */
void AlbenList::showOnlineSearch()
{
#ifdef ONLINE_SUPPORT
	OnlineSearchWindow *onlineSearchWindow = new OnlineSearchWindow();
	onlineSearchWindow->setAttribute(Qt::WA_DeleteOnClose);
	onlineSearchWindow->show();
#endif
}

