#include "albentable.h"
#include "ui_albentable.h"

#include "album.h"
#include "albendetail.h"
#include "albenlist.h"
#include "dataworker.h"
#include "definitions.h"
#include "server.h"
#include "titlelist.h"
#include "titletable.h"

#include <QList>
#include <QMap>
#include <QMessageBox>
#include <QTableWidgetItem>
#include <QVector>

#include <UmlauteWeb/Qt.hpp>

#ifdef ONLINE_SUPPORT
#include "onlinesearchwindow.h"
#endif

/**
 * @brief constructor, initializing UI
 * @param parent
 */
AlbenTable::AlbenTable(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::AlbenTable)
{
	ui->setupUi(this);
}

/**
 * @brief Destructor, destruct UI
 */
AlbenTable::~AlbenTable()
{
	delete ui;
}

/**
 * @brief showing @see AlbenDetail in 'add entry' mode
 */
void AlbenTable::addEntry()
{
	QString ean = "-1";
	DetailModi mode = DetailModi::MODE_ADD_ENTRY;
	AlbenDetail *albenDetail = new AlbenDetail();
	albenDetail->setAttribute(Qt::WA_DeleteOnClose);
	albenDetail->load(ean, mode);
}

/**
 * @brief changing the selectes entry by showing @see AlbenDetail in 'change' mode
 */
void AlbenTable::changeEntry()
{
	int rowIndex = ui->dataTable->currentItem()->row();
	QString ean = ui->dataTable->item(rowIndex, 3)->text();

	DetailModi mode = DetailModi::MODE_READONLY_IDENTIFYER;
	AlbenDetail *albenDetail = new AlbenDetail();
	albenDetail->setAttribute(Qt::WA_DeleteOnClose);
	albenDetail->load(ean, mode);
}

/**
 * @brief change to another view
 */
void AlbenTable::changeView()
{
	if (sender()->objectName() == "actionAlbenStd")
	{
		AlbenList *albenList = new AlbenList();
		albenList->setAttribute(Qt::WA_DeleteOnClose);
		albenList->show();
		//close();
	}

	if (sender()->objectName() == "actionTitelStd")
	{
		TitleList *titleList = new TitleList();
		titleList->setAttribute(Qt::WA_DeleteOnClose);
		titleList->show();
		//close();
	}

	if (sender()->objectName() == "actionTitelFull")
	{
		TitleTable *titleTable = new TitleTable();
		titleTable->setAttribute(Qt::WA_DeleteOnClose);
		titleTable->show();
		//close();
	}
	close();
}

/**
 * @brief user action: click on delete
 */
void AlbenTable::deleteEntry()
{
	int rowIndex = ui->dataTable->currentRow();
	QString ean = ui->dataTable->item(rowIndex, 3)->text();
	deleteEntry(ean);
}

/**
 * @brief delete entry from database
 * @param ean identify the entry to delete
 */
void AlbenTable::deleteEntry(QString ean)
{
	QMessageBox::StandardButton requestNo = QMessageBox::question(
												this,
												QApplication::translate("Definitions", PRODUCT_NAME),
												QApplication::translate("Definitions", ARE_YOU_SURE_DELETE_ENTRY),
												QMessageBox::Yes | QMessageBox::No);

	if (requestNo == QMessageBox::No) { return; }

	QMap<QString,QString> filter{};
	filter.insert("EAN", ean);

	bool deleteOK = Server::getInstance().database->deleteEntry(TABLE, filter);

	if (deleteOK)
	{
		QMessageBox::information(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					QApplication::translate("Definitions", DELETE_SUCCESS)
		);
	}
	else
	{
		QMessageBox msgBox;
		msgBox.setText(QApplication::translate("Definitions", DELETE_FAILED));
		msgBox.setWindowTitle(QApplication::translate("Definitions", PRODUCT_NAME));
		msgBox.setInformativeText(Server::getInstance().database->DatabaseWorker::getLastError());
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setIcon(QMessageBox::Icon::Critical);
		msgBox.exec();
	}
	showAllEntrys();
}

/**
 * @brief disables options change, delete, modify
 */
void AlbenTable::disableButtons()
{
	ui->actionChange->setEnabled(false);
	ui->actionDelete->setEnabled(false);
	ui->actionView->setEnabled(false);
}

/**
 * @brief  enables options change, delete, modify
 */
void AlbenTable::enableButtons()
{
	ui->actionChange->setEnabled(true);
	ui->actionDelete->setEnabled(true);
	ui->actionView->setEnabled(true);
}

/**
 * @brief displays the given \p albums
 * @param albums the entrys to display in lstValues
 */
void AlbenTable::output(QList<Album*> *albums)
{
	ui->dataTable->clearContents();
	ui->dataTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);

	if (albums != nullptr)
	{
		ui->dataTable->setRowCount(albums->count());
		ui->dataTable->setColumnCount(8);
	
		int rowCounter = 0;
	
		for (auto &&album : *albums)
		{
			QString title = album->title;
			Umlaute_Web::Qt::decode(title);
			QTableWidgetItem *titleItem = new QTableWidgetItem(title);
			ui->dataTable->setItem(rowCounter, 0, titleItem);
	
			QString ipret = album->interpret;
			Umlaute_Web::Qt::decode(ipret);
			QTableWidgetItem *ipretItem = new QTableWidgetItem(ipret);
			ui->dataTable->setItem(rowCounter, 1, ipretItem);
	
			QString year = QString::number(album->year);
			QTableWidgetItem *yearItem = new QTableWidgetItem(year);
			ui->dataTable->setItem(rowCounter, 2, yearItem);
	
			QString ean = album->ean;
			QTableWidgetItem *eanItem = new QTableWidgetItem(ean);
			ui->dataTable->setItem(rowCounter, 3, eanItem);
	
			QString grene = album->grene;
			Umlaute_Web::Qt::decode(grene);
			QTableWidgetItem *greneItem = new QTableWidgetItem(grene);
			ui->dataTable->setItem(rowCounter, 4, greneItem);
	
			QString type = album->isCD_DVD ? "CD + DVD" : "CD";
			QTableWidgetItem *typeItem = new QTableWidgetItem(type);
			ui->dataTable->setItem(rowCounter, 5, typeItem);
	
			QString loc = album->location;
			Umlaute_Web::Qt::decode(loc);
			QTableWidgetItem *locItem = new QTableWidgetItem(loc);
			ui->dataTable->setItem(rowCounter, 6, locItem);
	
			QString own = album->owner;
			Umlaute_Web::Qt::decode(own);
			QTableWidgetItem *ownItem = new QTableWidgetItem(own);
			ui->dataTable->setItem(rowCounter, 7, ownItem);
	
			rowCounter++;
		}
	}
	ui->dataTable->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

/**
 * @brief search for a specific entry
 * @param creteria the row to look in
 * @param value the value to search for
 */
void AlbenTable::search(QMap<QString, QString> *searchFilter)
{
	QList<QVector<QString>*> *res = nullptr;

	res = Server::getInstance().database->searchEntries(TABLE, searchFilter, nullptr, SORTING_CRETERIA);

	if (res == nullptr)
	{
		QMessageBox::critical(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					Server::getInstance().database->DatabaseWorker::getLastError(),
					QMessageBox::Ok);
		return;
	}

	QList<Album*> *results = DataWorker::getAlbums(res);
	output(results);

	if (searchFilter != nullptr) { delete searchFilter; }
	delete res;
	delete results;
}

/**
 * @brief filteroptions changed => new search
 */
void AlbenTable::searchCriterionChanged()
{
	search(DataWorker::createSearchFilterForAlbum(ui->comboSearch1->currentIndex(),
												  ui->comboSearch2->currentIndex(),
												  ui->txtSearch1->text(),
												  ui->txtSearch2->text()));
}

/**
 * @brief check if buttons need to be activated
 * @param current the current selected item
 * @param previous the previous selected item
 */
void AlbenTable::selectedDataChanged(QTableWidgetItem *current, QTableWidgetItem *previous)
{
	int index = current != nullptr ? current->row() : -1;
	if (index > -1) { enableButtons();}
}

/**
 * @brief show all database entrys
 */
void AlbenTable::showAllEntrys()
{
	search();
}

/**
 * @brief doing stuff while window ist beginning to be visible
 * @param ev the underlaying event
 */
void AlbenTable::showEvent(QShowEvent *ev)
{
	QMainWindow::showEvent(ev);
	QMetaObject::invokeMethod(this, "windowLoaded", Qt::ConnectionType::QueuedConnection);
}

/**
 * @brief view details of the selected entry by showing @see AlbenDetail in 'readonly' mode
 */
void AlbenTable::viewEntry()
{
	int rowIndex = ui->dataTable->currentItem()->row();
	QString ean = ui->dataTable->item(rowIndex, 3)->text();

	DetailModi mode = DetailModi::MODE_READONLY_ALL;
	AlbenDetail *albenDetail = new AlbenDetail();
	albenDetail->setAttribute(Qt::WA_DeleteOnClose);
	albenDetail->load(ean, mode);
}

/**
 * @brief when window is loading show all entrys
 */
void AlbenTable::windowLoaded()
{
	disableButtons();
	showAllEntrys();
}

/**
 * @brief Show the online search window
 */
void AlbenTable::showOnlineSearch()
{
#ifdef ONLINE_SUPPORT
	OnlineSearchWindow *onlineWindow = new OnlineSearchWindow();
	onlineWindow->setAttribute(Qt::WA_DeleteOnClose);
	onlineWindow->show();
#endif
}
