#include "splashscreen.h"
#include "version.h"

#include <QApplication>
#include <QLocale>
#include <QStandardPaths>
#include <QTranslator>

#include <iostream>

int main(int argc, char *argv[])
{
	QApplication app(argc, argv);
	app.setQuitOnLastWindowClosed(true);

	QString appTrPath;
	QString libTrPath;

#if defined(QT_DEBUG)
		appTrPath = app.applicationDirPath() + "/";
#else
#if defined(Q_OS_WIN)
		appTrPath = app.applicationDirPath() + "/translations";
		libTrPath = appTrPath;
#else
	appTrPath = QStandardPaths::locate(QStandardPaths::AppDataLocation, "i18n", QStandardPaths::LocateDirectory);
	libTrPath = appTrPath + "/../../MySqlWorker/i18n";
#endif
#endif

		QTranslator appTranslator;
		QTranslator libTranslator;

		bool appTranslationInstalled = false;
		bool libTranslationInstalled = false;

#if defined(Q_OS_WIN)
		QTranslator baseTranslator;
		bool baseTranslationInstalled = false;
#endif

		const QStringList uiLanguages = QLocale::system().uiLanguages();
		for (const QString &locale : uiLanguages)
		{
			const QString localeName = QLocale(locale).name();

#if defined(Q_OS_WIN)
			libTrPath = appTrPath;
			if(!baseTranslationInstalled)
			{
				const QString baseName = "/qt_" + localeName;
				bool baseTranslationLoaded = baseTranslator.load(appTrPath + baseName);
				if(baseTranslationLoaded)
				{
					app.installTranslator(&baseTranslator);
					baseTranslationInstalled = true;
				}
			}
#endif

			if (!appTranslationInstalled)
			{
				const QString baseNameApp = QString("/") + PROJECT_NAME + "_" + localeName;
				bool appTranslationLoaded = appTranslator.load(appTrPath + baseNameApp);
				if (appTranslationLoaded)
				{
					app.installTranslator(&appTranslator);
					appTranslationInstalled = true;
				}
			}

			if (!libTranslationInstalled)
			{
				const QString baseNameLibLogView = "/MySqlWorker_" + localeName;
				bool libTranslationLoaded = libTranslator.load(libTrPath + baseNameLibLogView);
				if (libTranslationLoaded)
				{
					app.installTranslator(&libTranslator);
					libTranslationInstalled = true;
				}

			}
		}

	app.setOrganizationName("catfood");
	app.setApplicationName(PROJECT_NAME);
	app.setOrganizationDomain("de.catfood");

	SplashScreen *w = new SplashScreen();
	w->setAttribute(Qt::WA_DeleteOnClose);
	w->setWindowFlags(Qt::SplashScreen);
	w->show();

	return app.exec();
}
