#include "onlinesearchwindow.h"
#include "ui_onlinesearchwindow.h"

#include "definitions.h"
#include "server.h"

#include <musicbrainz5/ArtistCredit.h>
#include <musicbrainz5/HTTPFetch.h>
#include <musicbrainz5/NameCredit.h>
#include <musicbrainz5/NameCreditList.h>
#include <musicbrainz5/Query.h>
#include <musicbrainz5/Release.h>
#include <musicbrainz5/Recording.h>
#include <musicbrainz5/RecordingList.h>

#include <SqlWorker/DatabaseWorker.hpp>

#include <QList>
#include <QMap>
#include <QMessageBox>
#include <QPair>
#include <QSet>
#include <QString>
#include <QVector>

#include <UmlauteWeb/Qt.hpp>

/**
 * @brief Initialize window
 * @param parent The parent widget
 */
OnlineSearchWindow::OnlineSearchWindow(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::OnlineSearchWindow)
{
	ui->setupUi(this);
	ui->table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

/**
 * @brief Destroy window
 */
OnlineSearchWindow::~OnlineSearchWindow()
{
	delete ui;
}

void OnlineSearchWindow::btnSearchClicked()
{
	ui->table->clearContents();

	MusicBrainz5::CQuery query("releasequery-1.0");
	MusicBrainz5::CQuery::tParamMap params;

	// params["query"] = "\"<Interpret>\" AND \"<Title>\""
	params["query"] = "\""
					  + ui->txtInterpret->text().toStdString()
					  + "\""
					  + " AND "
					  + "\""
					  + ui->txtTitle->text().toStdString()
					  + "\"";


	ui->statusbar->showMessage(QApplication::translate("Definitions", ONLINE_SEARCH_MESSAGE));

	QSet<QPair<QString,QString>> entriesFound{};
	bool queryOk = true;

	try
	{
		MusicBrainz5::CMetadata metadata = query.Query("recording", "", "", params);

		// try to get a list of recordings
		MusicBrainz5::CRecordingList *recordingList = metadata.RecordingList();
		if (recordingList != nullptr)
		{
			// for each rexording in the list do
			for (int i = 0; i < recordingList->NumItems(); ++i)
			{
				MusicBrainz5::CRecording *recording = recordingList->Item(i);

				// try to get the release list of this recording
				if (recording != nullptr && recording->ReleaseList())
				{
					// for each relese in this list do
					for (int j = 0; j < recording->ReleaseList()->NumItems(); ++j)
					{
						MusicBrainz5::CRelease *release = recording->ReleaseList()->Item(j);

						/*
						 * get the list of artists
						 *   - iff there are more than one: select "Various Artists" as search string for the database
						 *   - else select the artist
						 *
						 * (if the following code causes a segfault we must check more carefully (smaller steps))
						 */
						MusicBrainz5::CArtistCredit *artistCredit = release->ArtistCredit();
						if (artistCredit != nullptr)
						{

							//if (release->ArtistCredit())
							QString artist = "";

							if (int artistNumber = release->ArtistCredit()->NameCreditList()->NumItems() > 0)
							{
								artist = artistNumber > 1
										 ? "Various Artists"
										 : QString::fromStdString(
											   release->ArtistCredit()->NameCreditList()->Item(0)->Name());
							}
							QString title = QString::fromStdString(recording->ReleaseList()->Item(j)->Title());
							// store the (title, artist) pair in the set (each pair is unique)
							entriesFound.insert(QPair<QString,QString>{title, artist});
						}
					}
				}
			}
		}
		else { queryOk = false; }
	}
	catch (MusicBrainz5::CConnectionError& Error)
	{
		std::cout << "Connection Exception: '" << Error.what() << "'" << std::endl;
		std::cout << "LastResult: " << query.LastResult() << std::endl;
		std::cout << "LastHTTPCode: " << query.LastHTTPCode() << std::endl;
		std::cout << "LastErrorMessage: " << query.LastErrorMessage() << std::endl;
		queryOk = false;
	}
	catch (MusicBrainz5::CTimeoutError& Error)
	{
		std::cout << "Timeout Exception: '" << Error.what() << "'" << std::endl;
		std::cout << "LastResult: " << query.LastResult() << std::endl;
		std::cout << "LastHTTPCode: " << query.LastHTTPCode() << std::endl;
		std::cout << "LastErrorMessage: " << query.LastErrorMessage() << std::endl;
		queryOk = false;
	}
	catch (MusicBrainz5::CAuthenticationError& Error)
	{
		std::cout << "Authentication Exception: '" << Error.what() << "'" << std::endl;
		std::cout << "LastResult: " << query.LastResult() << std::endl;
		std::cout << "LastHTTPCode: " << query.LastHTTPCode() << std::endl;
		std::cout << "LastErrorMessage: " << query.LastErrorMessage() << std::endl;
		queryOk = false;
	}
	catch (MusicBrainz5::CFetchError& Error)
	{
		std::cout << "Fetch Exception: '" << Error.what() << "'" << std::endl;
		std::cout << "LastResult: " << query.LastResult() << std::endl;
		std::cout << "LastHTTPCode: " << query.LastHTTPCode() << std::endl;
		std::cout << "LastErrorMessage: " << query.LastErrorMessage() << std::endl;
		queryOk = false;
	}
	catch (MusicBrainz5::CRequestError& Error)
	{
		std::cout << "Request Exception: '" << Error.what() << "'" << std::endl;
		std::cout << "LastResult: " << query.LastResult() << std::endl;
		std::cout << "LastHTTPCode: " << query.LastHTTPCode() << std::endl;
		std::cout << "LastErrorMessage: " << query.LastErrorMessage() << std::endl;
		queryOk = false;
	}
	catch (MusicBrainz5::CResourceNotFoundError& Error)
	{
		std::cout << "ResourceNotFound Exception: '" << Error.what() << "'" << std::endl;
		std::cout << "LastResult: " << query.LastResult() << std::endl;
		std::cout << "LastHTTPCode: " << query.LastHTTPCode() << std::endl;
		std::cout << "LastErrorMessage: " << query.LastErrorMessage() << std::endl;
		queryOk = false;
	}

	if (queryOk)
	{
		const QString table = "Albums";
		const QString sorting = "Title";
		const QVector<QString> columns{"Title", "Interpret", "Location"};

		int rowCount = 0;
		ui->table->setColumnCount(4);
		ui->table->setRowCount(entriesFound.count());

		ui->statusbar->showMessage(QApplication::translate("Definitions", LOCAL_SEARCH_MESSAGE));


		for (auto item : qAsConst(entriesFound))
		{

			QMap<QString,QString> filter{};
			Umlaute_Web::Qt::encode(item.first);
			Umlaute_Web::Qt::encode(item.second);
			filter.insert("Title", item.first);
			filter.insert("Interpret", item.second);

			QList<QVector<QString>*> *result =
					Server::getInstance().database->searchEntries(table,
																  &filter,
																  &columns,
																  sorting);

			// revert encoding
			Umlaute_Web::Qt::decode(item.first);
			Umlaute_Web::Qt::decode(item.second);

			/*
			 * If there is an error in querying the database we can get a nullpointer back...
			 */
			if (result == nullptr || result->count() == 0)
			{
				// nothing found for this
				ui->table->setItem(rowCount, 0, new QTableWidgetItem(item.first));
				ui->table->setItem(rowCount, 1, new QTableWidgetItem(item.second));
				ui->table->setItem(rowCount, 2, new QTableWidgetItem(tr("No")));
				ui->table->setItem(rowCount, 3, new QTableWidgetItem(""));
				rowCount++;
			}
			else
			{
				/*
				 * Since MySqlWorker::DatabaseWorker is a "like search" we cannto guarantee here, that we get only one
				 * entry back from the database. for this reason we iterate over all possible results from the database
				 * (and not just displaying the first value).
				 */
				for (auto &resultItem : *result)
				{
					if (resultItem->at(0) == item.first && resultItem->at(1) == item.second)
					{
						ui->table->setItem(rowCount, 0, new QTableWidgetItem(item.first));
						ui->table->setItem(rowCount, 1, new QTableWidgetItem(item.second));
						ui->table->setItem(rowCount, 2, new QTableWidgetItem(tr("Yes")));
						ui->table->setItem(rowCount, 3, new QTableWidgetItem(resultItem->at(2)));
						rowCount++;
					}
					// else: title from DB not matching search term exactly -> ignore it
				}
			}
		}
		ui->statusbar->showMessage(QApplication::translate("Definitions", SEARCH_SUCCESS));
	}
	else
	{
		QMessageBox::critical(this,
							  QApplication::translate("Definitions", PRODUCT_NAME),
							  QApplication::translate("Definitions", ONLINE_FAILED)
							  .arg(QString::fromStdString(query.LastErrorMessage()))
		);
		ui->statusbar->showMessage(QApplication::translate("Definitions", ONLINE_FAILED));
	}
}

void OnlineSearchWindow::btnAbortClicked()
{
	ui->txtInterpret->clear();
	ui->txtTitle->clear();
	ui->statusbar->clearMessage();
}


