#include "titlelist.h"
#include "ui_titlelist.h"

#include "albenlist.h"
#include "albentable.h"
#include "dataworker.h"
#include "definitions.h"
#include "detailmodi.h"
#include "smalltitle.h"
#include "server.h"
#include "titledetail.h"
#include "titletable.h"

#include <SqlWorker/DatabaseWorker.hpp>

#include <QList>
#include <QMessageBox>
#include <QString>
#include <QVector>

#include <UmlauteWeb/Qt.hpp>

/**
 * @brief constructor, setup the UI
 * @param parent
 */
TitleList::TitleList(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::TitleList)
{
	ui->setupUi(this);
}

/**
 * @brief destructor, delete the ui
 */
TitleList::~TitleList()
{
	delete ui;
}

/**
 * @brief showing @see AlbenDetail in 'add entry' mode
 */
void TitleList::addEntry()
{
	int id = -1;
	DetailModi mode = DetailModi::MODE_ADD_ENTRY;
	TitleDetail *titleDetail = new TitleDetail();
	titleDetail->setAttribute(Qt::WA_DeleteOnClose);
	titleDetail->load(id, mode);
}

/**
 * @brief changing the selectes entry by showing <see cref="AlbenDetail"/> in 'change' mode
 */
void TitleList::changeEntry()
{
	int index = ui->lstValues->currentRow();
	int id = ids[index];
	DetailModi mode = DetailModi::MODE_READONLY_IDENTIFYER;
	TitleDetail *titleDetail = new TitleDetail();
	titleDetail->setAttribute(Qt::WA_DeleteOnClose);
	titleDetail->load(id, mode);
}

/**
 * @brief change to another view
 */
void TitleList::changeView()
{
	if (sender()->objectName() == "actionAlbenStd")
	{
		AlbenList *albenList = new AlbenList();
		albenList->setAttribute(Qt::WA_DeleteOnClose);
		albenList->show();
		//close();
	}

	if (sender()->objectName() == "actionAlbenFull")
	{
		AlbenTable *albenTable = new AlbenTable();
		albenTable->setAttribute(Qt::WA_DeleteOnClose);
		albenTable->show();
		//close();
	}

	if (sender()->objectName() == "actionTitelFull")
	{
		TitleTable *titleTable = new TitleTable();
		titleTable->setAttribute(Qt::WA_DeleteOnClose);
		titleTable->show();
		//close();
	}
	close();
}

/**
 * @brief user action: click on delete
 */
void TitleList::deleteEntry()
{
	int index = ui->lstValues->currentRow();
	int id = ids[index];
	deleteEntry(id);
}

/**
 * @brief delete entry from database
 * @param id identify the entry to delete
 */
void TitleList::deleteEntry(int id)
{
	QMessageBox::StandardButton requestNo = QMessageBox::question(
												this,
												QApplication::translate("Definitions", PRODUCT_NAME),
												QApplication::translate("Definitions", ARE_YOU_SURE_DELETE_ENTRY)
											);

	if (requestNo == QMessageBox::StandardButton::No) { return; }

	QMap<QString,QString> filter{};
	filter.insert("ID", QString::number(id));
	bool deleteOK = Server::getInstance().database->DatabaseWorker::deleteEntry(TABLE, filter);

	if (deleteOK)
	{
		QMessageBox::information(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					QApplication::translate("Definitions", DELETE_SUCCESS)
		);
	}
	else
	{
		QMessageBox msgBox;
		msgBox.setText(QApplication::translate("Definitions", DELETE_FAILED));
		msgBox.setWindowTitle(QApplication::translate("Definitions", PRODUCT_NAME));
		msgBox.setInformativeText(Server::getInstance().database->DatabaseWorker::getLastError());
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setIcon(QMessageBox::Icon::Critical);
		msgBox.exec();
	}
	search();
}

/**
 * @brief disables options change, delete, modify
 */
void TitleList::disableButtons()
{
	ui->actionChange->setEnabled(false);
	ui->actionDelete->setEnabled(false);
	ui->actionView->setEnabled(false);
	ui->btnChange->setEnabled(false);
	ui->btnDel->setEnabled(false);
	ui->btnView->setEnabled(false);
}

/**
 * @brief enables options change, delete, modify
 */
void TitleList::enableButtons()
{
	ui->actionChange->setEnabled(true);
	ui->actionDelete->setEnabled(true);
	ui->actionView->setEnabled(true);
	ui->btnChange->setEnabled(true);
	ui->btnDel->setEnabled(true);
	ui->btnView->setEnabled(true);
}

/**
 * @brief check if buttons need to be activated
 * @param index the current index
 */
void TitleList::lstSelectedIndexChanged(int index)
{
	if (index > -1) { enableButtons(); }
	else { disableButtons(); }
}

/**
 * @brief displays the given \p titles
 * @param titles the entrys to display in lstValues
 */
void TitleList::output(QList<SmallTitle*> *titles)
{
	ui->lstValues->clear();
	ids.clear();
	if (titles != nullptr)
	{
		for (auto &&title : *titles)
		{
			QString titleString = title->title;
			Umlaute_Web::Qt::decode(titleString);
	
			QString albumString = title->album;
			Umlaute_Web::Qt::decode(albumString);
	
			QString interpretString = title->interpret;
			Umlaute_Web::Qt::decode(interpretString);
	
			QString outputString = titleString + " # " + albumString + " # " + interpretString;
			ui->lstValues->addItem(outputString);
			ids.append(title->id);
		}
	}
}

/**
 * @brief search for a specific entry
 * @param criterion the row to look in
 * @param value the value to search for
 */
void TitleList::search(QMap<QString,QString> *filter)
{
	QVector<QString> columns{"ID", "Title", "Album", "Interpret"};
	QList<QVector<QString>*> *res;

	res = Server::getInstance().database->searchEntries(TABLE, filter, &columns, SORTING_CRETERIA);


	// error from MySQLWorker?
	if (res == nullptr)
	{
		QMessageBox::critical(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					Server::getInstance().database->DatabaseWorker::getLastError(),
					QMessageBox::Ok);
		return;
	}
	QList<SmallTitle*> *results = DataWorker::getSmallTitles(res);
	output(results);

	delete results;
	delete res;

}

/**
 * @brief filteroptions changed => new search
 */
void TitleList::searchCriterionChanged()
{
	search(DataWorker::createSearchFilterForTitle(ui->comboSearch1->currentIndex(),
												  ui->comboSearch2->currentIndex(),
												  ui->txtSearch1->text(),
												  ui->txtSearch2->text()));
}

/**
 * @brief doing stuff while window ist beginning to be visible
 * @param ev the underlaying event
 */
void TitleList::showEvent(QShowEvent *ev)
{
	QMainWindow::showEvent(ev);
	QMetaObject::invokeMethod(this, "windowLoaded", Qt::ConnectionType::QueuedConnection);
}

/**
 * @brief view details of the selected entry by showing @see AlbenDetail in 'readonly' mode
 */
void TitleList::viewEntry()
{
	int index = ui->lstValues->currentRow();
	int id = ids[index];
	DetailModi mode = DetailModi::MODE_READONLY_ALL;
	TitleDetail *titleDetail = new TitleDetail();
	titleDetail->setAttribute(Qt::WA_DeleteOnClose);
	titleDetail->load(id, mode);
}

/**
 * @brief when window is loading show all entrys
 */
void TitleList::windowLoaded()
{
	search();
}

