#include "titletable.h"
#include "ui_titletable.h"

#include "albenlist.h"
#include "albentable.h"
#include "dataworker.h"
#include "definitions.h"
#include "detailmodi.h"
#include "server.h"
#include "title.h"
#include "titledetail.h"
#include "titlelist.h"

#include <SqlWorker/DatabaseWorker.hpp>

#include <QList>
#include <QMessageBox>
#include <QString>
#include <QTableWidgetItem>
#include <QVector>

#include <UmlauteWeb/Qt.hpp>

/**
 * @brief constructor, initializing UI
 * @param parent
 */
TitleTable::TitleTable(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::TitleTable)
{
	ui->setupUi(this);
}

/**
 * @brief destructor, destructing UI
 */
TitleTable::~TitleTable()
{
	delete ui;
}

/**
 * @brief showing @see TitleDetail in 'add entry' mode
 */
void TitleTable::addEntry()
{
	int id = -1;
	DetailModi mode = DetailModi::MODE_ADD_ENTRY;
	TitleDetail *titleDetail = new TitleDetail();
	titleDetail->setAttribute(Qt::WA_DeleteOnClose);
	titleDetail->load(id, mode);
}

/**
 * @brief changing the selectes entry by showing @see TitleDetail in 'change' mode
 */
void TitleTable::changeEntry()
{
	int rowIndex = ui->dataTable->currentRow();
	QString idString = ui->dataTable->item(rowIndex, 0)->text();
	short id = idString.toShort();

	DetailModi mode = DetailModi::MODE_READONLY_IDENTIFYER;
	TitleDetail *titleDetail = new TitleDetail();
	titleDetail->setAttribute(Qt::WA_DeleteOnClose);
	titleDetail->load(id, mode);
}

/**
 * @brief change to another view
 */
void TitleTable::changeView()
{
	if (sender()->objectName() == "actionAlbenStd")
	{
			AlbenList *albenList = new AlbenList();
			albenList->setAttribute(Qt::WA_DeleteOnClose);
			albenList->show();
			//close();
	}

	if (sender()->objectName() == "actionAlbenFull")
	{
			AlbenTable *titleTable = new AlbenTable();
			titleTable->setAttribute(Qt::WA_DeleteOnClose);
			titleTable->show();
			//close();
	}

	if (sender()->objectName() == "actionTitelStd")
	{
			TitleList *titleList = new TitleList();
			titleList->setAttribute(Qt::WA_DeleteOnClose);
			titleList->show();
			//close();
	}

	close();
}

/**
 * @brief  user action: click on delete
 */
void TitleTable::deleteEntry()
{
	int rowIndex = ui->dataTable->currentRow();
	short id = ui->dataTable->item(rowIndex, 0)->text().toShort();
	deleteEntry(id);
}

/**
 * @brief delete entry from database
 * @param id identify the entry to delete
 */
void TitleTable::deleteEntry(int id)
{
	QMessageBox::StandardButton requestNo = QMessageBox::question(
													this,
													QApplication::translate("Definitions", PRODUCT_NAME),
													QApplication::translate("Definitions", ARE_YOU_SURE_DELETE_ENTRY),
													QMessageBox::Yes | QMessageBox::No);

	if (requestNo == QMessageBox::No) { return; }

	QMap<QString,QString> filter{};
	filter.insert("ID", QString::number(id));

	bool deleteOK = Server::getInstance().database->deleteEntry(TABLE, filter);
	if (deleteOK)
	{
			QMessageBox::information(
						this,
						QApplication::translate("Definitions", PRODUCT_NAME),
						QApplication::translate("Definitions", DELETE_SUCCESS)
			);
	}
	else
	{
		QMessageBox msgBox;
		msgBox.setText(QApplication::translate("Definitions", DELETE_FAILED));
		msgBox.setWindowTitle(QApplication::translate("Definitions", PRODUCT_NAME));
		msgBox.setInformativeText(Server::getInstance().database->DatabaseWorker::getLastError());
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setIcon(QMessageBox::Icon::Critical);
		msgBox.exec();
	}
	search();
}

/**
 * @brief disables options change, delete, modify
 */
void TitleTable::disableButtons()
{
	ui->actionChange->setEnabled(false);
	ui->actionDelete->setEnabled(false);
	ui->actionView->setEnabled(false);
}

/**
 * @brief  enables options change, delete, modify
 */
void TitleTable::enableButtons()
{
	ui->actionChange->setEnabled(true);
	ui->actionDelete->setEnabled(true);
	ui->actionView->setEnabled(true);
}

/**
 * @brief displays the given \p titles
 * @param titles the entrys to display in lstValues
 */
void TitleTable::output(QList<Title*> *titles)
{
	ui->dataTable->clearContents();
	ui->dataTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);

	if (titles != nullptr)
	{
		ui->dataTable->setRowCount(titles->count());
		ui->dataTable->setColumnCount(8);
	
		int rowCounter = 0;
	
		for (auto &&title : *titles)
		{
			QString titleString = title->title;
			Umlaute_Web::Qt::decode(titleString);
			QTableWidgetItem *titleItem = new QTableWidgetItem(titleString);
			ui->dataTable->setItem(rowCounter, 0, titleItem);
	
			QString ipret = title->interpret;
			Umlaute_Web::Qt::decode(ipret);
			QTableWidgetItem *ipretItem = new QTableWidgetItem(ipret);
			ui->dataTable->setItem(rowCounter, 1, ipretItem);
	
			QString album = title->album;
			Umlaute_Web::Qt::decode(album);
			QTableWidgetItem *albumItem = new QTableWidgetItem(album);
			ui->dataTable->setItem(rowCounter, 2, albumItem);

			QString year = QString::number(title->year);
			QTableWidgetItem *yearItem = new QTableWidgetItem(year);
			ui->dataTable->setItem(rowCounter, 3, yearItem);
	
			QString grene = title->grene;
			Umlaute_Web::Qt::decode(grene);
			QTableWidgetItem *greneItem = new QTableWidgetItem(grene);
			ui->dataTable->setItem(rowCounter, 4, greneItem);

			QString format = title->format;
			Umlaute_Web::Qt::decode(format);
			QTableWidgetItem *formatItem = new QTableWidgetItem(format);
			ui->dataTable->setItem(rowCounter, 5, formatItem);
	
	
			QString owner = title->owner;
			Umlaute_Web::Qt::decode(owner);
			QTableWidgetItem *ownerItem = new QTableWidgetItem(owner);
			ui->dataTable->setItem(rowCounter, 6, ownerItem);
	
			QString id = QString::number(title->id);
			QTableWidgetItem *idItem = new QTableWidgetItem(id);
			ui->dataTable->setItem(rowCounter, 7, idItem);
	
			rowCounter++;
		}
	}
	ui->dataTable->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

/**
 * @brief search for a specific entry
 * @param criterion the row to look in
 * @param value the value to search for
 */
void TitleTable::search(QMap<QString, QString> *filter)
{
	QList<QVector<QString>*> *res = nullptr;

	res = Server::getInstance().database->searchEntries(TABLE, filter, nullptr, SORTING_CRETERIA);

	if (res == nullptr)
	{
		QMessageBox::critical(
					this,
					QApplication::translate("Definitions", PRODUCT_NAME),
					Server::getInstance().database->DatabaseWorker::getLastError(),
					QMessageBox::Ok);
		return;
	}

	QList<Title*> *results = DataWorker::getTitles(res);
	output(results);

	if (filter != nullptr) { delete filter; }
	delete res;
	delete results;
}

/**
 * @brief  filteroptions changed => new search
 */
void TitleTable::searchCriterionChanged()
{
	search(DataWorker::createSearchFilterForTitle(ui->comboSearch1->currentIndex(),
												  ui->comboSearch2->currentIndex(),
												  ui->txtSearch1->text(),
												  ui->txtSearch2->text()));
}

/**
 * @brief check if buttons need to be activated
 * @param current the current selected entry
 * @param previous
 */
void TitleTable::selectedDataChanged(QTableWidgetItem *current, QTableWidgetItem *previous)
{
	int index = current != nullptr ? current->row() : -1;
	if (index > -1) { enableButtons(); }
}

/**
 * @brief doing stuff while window ist beginning to be visible
 * @param ev the underlaying event
 */
void TitleTable::showEvent(QShowEvent *ev)
{
	QMainWindow::showEvent(ev);
	QMetaObject::invokeMethod(this, "windowLoaded", Qt::ConnectionType::QueuedConnection);
}

/**
 * @brief view details of the selected entry by showing @see TitleDetail in 'readonly' mode
 */
void TitleTable::viewEntry()
{
	short id = ui->dataTable->item(ui->dataTable->currentRow(), 0)->text().toShort();
	DetailModi mode = DetailModi::MODE_READONLY_ALL;
	TitleDetail *titleDetail = new TitleDetail();
	titleDetail->setAttribute(Qt::WA_DeleteOnClose);
	titleDetail->load(id, mode);
}

/**
 * @brief when window is loading show all entrys
 */
void TitleTable::windowLoaded()
{
	disableButtons();
	search();
}
